/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.html.demo.chess;

import net.java.html.json.ComputedProperty;
import net.java.html.json.Model;
import net.java.html.json.Property;


/** Communication protocols to talk to the server.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
final class Communication {
    @Model(className = "Request", properties = {
        @Property(name = "msg", type = MsgType.class),
        @Property(name = "username", type = String.class),
        @Property(name = "password", type = String.class),
        @Property(name = "gameId", type = String.class),
        @Property(name = "color", type = Color.class),
        @Property(name = "from", type = String.class),
        @Property(name = "to", type = String.class),
        @Property(name = "observer", type = boolean.class),
    })
    static class RequestModel {
    }

    @Model(className = "BoardRep", properties = {
        @Property(name = "whites", type = String.class, array = true),
        @Property(name = "blacks", type = String.class, array = true),
    })
    static class BoardRepModel {
    }

    @Model(className = "Response", properties = {
        @Property(name = "msg", type = String.class),
        @Property(name = "games", array = true, type = Game.class),
        @Property(name = "board", type = BoardRep.class),
        @Property(name = "turn", type = Color.class),
        @Property(name = "color", type = Color.class),
        @Property(name = "gameId", type = String.class),
        @Property(name = "error", type = Err.class),
        @Property(name = "alert", type = Alert.class),
        @Property(name = "status", type = String.class),
        @Property(name = "summary", type = String.class),
        @Property(name = "moves", type = String.class, array = true),
        @Property(name = "from", type = String.class),
        @Property(name = "to", type = String.class),
        @Property(name = "check", type = CheckResult.class),
        @Property(name = "whitePlayer", type = String.class),
        @Property(name = "blackPlayer", type = String.class),
    })
    static class ResponseModel {
        @ComputedProperty static Color nextTurn(Color turn, Alert alert) {
            if (alert == null || !alert.getType().isTerminal()) {
                return turn;
            } else {
                return null;
            }
        }
    }

    static enum CheckResult {
        VALID, INVALID, NOT_REGISTERED
    };
    
    @Model(className = "Err", properties = {
        @Property(name = "code", type = int.class),
        @Property(name = "message", type = String.class),
    })
    static class ErrorModel {
    }

    @Model(className = "Alert", properties = {
        @Property(name = "type", type = AlertType.class),
        @Property(name = "message", type = String.class),
    })
    static class AlertModel {
    }

    static enum AlertType {
        CHECKMATE, CHECK, DRAW;

        public boolean isTerminal() {
            return this == CHECKMATE || this == DRAW;
        }
    };
    
}
