/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.html.demo.chess;

import java.util.List;
import net.java.html.json.ComputedProperty;
import net.java.html.json.Model;
import net.java.html.json.ModelOperation;
import net.java.html.json.Property;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Model(className = "Games", properties = {
    @Property(name = "own", array = true, type = Game.class),
    @Property(name = "other", array = true, type = Game.class),
    @Property(name = "selectedColor", type = String.class),
})
class GamesModel {
    @Model(className = "Game", properties = {
        @Property(name = "gameId", type=String.class),
        @Property(name = "summary", type=String.class),
        @Property(name = "open", type=boolean.class),
        @Property(name = "whitePlayer", type=String.class),
        @Property(name = "blackPlayer", type=String.class)
    })
    static class GameImpl {
        @ComputedProperty static String description(String gameId, String summary) {
            return summary != null ? summary : gameId;
        }
        
        @ComputedProperty static boolean pendingPlayer(String whitePlayer, String blackPlayer) {
            return whitePlayer == null || blackPlayer == null;
        }
    }
    
    
    @ComputedProperty static boolean anyOwnGame(List<Game> own) {
        return !own.isEmpty();
    }

    @ComputedProperty static boolean anyOtherGame(List<Game> other) {
        return !other.isEmpty();
    }
    
    @ModelOperation static void listGames(Games g, List<Game> games, String username) {
        g.getOwn().clear();
        g.getOther().clear();
        int cnt = 0;
        final int maxGames = 30;
        for (Game game : games) {
            if (
                username.equals(game.getWhitePlayer()) || 
                username.equals(game.getBlackPlayer())
            ) {
                g.getOwn().add(game);
            } else {
                if (cnt++ < maxGames) {
                    g.getOther().add(game);
                }
            }
        }
    }
}
